@@ -11,8 +11,6 @@
 #define PASSWORD_H
 #include "lib/timer.h"
 
-#define MD5_AUTH_SIZE 16
-
 struct password_item {
   node n;
   char *password;
@@ -24,6 +22,7 @@ extern struct password_item *last_password_item;
 
 struct password_item *password_find(list *l, int first_fit);
 struct password_item *password_find_by_id(list *l, int id);
+struct password_item *password_find_by_value(list *l, char *pass, uint size);
 
 static inline int password_verify(struct password_item *p1, char *p2, uint size)
 {
